@@ -20,11 +20,26 @@ from django.http import HttpResponse
 from django.utils.unittest import skipUnless
 
 import reversion
-from reversion.revisions import RegistrationError, RevisionManager, RevisionManagementError
+from reversion.revisions import RegistrationError, RevisionManager
 from reversion.models import Revision, Version, VERSION_ADD, VERSION_CHANGE, VERSION_DELETE
 from reversion.middleware import RevisionMiddleware
 
 
+ZERO = datetime.timedelta(0)
+
+
+class UTC(datetime.tzinfo):
+    """UTC"""
+
+    def utcoffset(self, dt):
+        return ZERO
+
+    def tzname(self, dt):
+        return "UTC"
+
+    def dst(self, dt):
+        return ZERO
+
 class ReversionTestModelBase(models.Model):
 
     name = models.CharField(
@@ -240,6 +255,28 @@ class ApiTest(RevisionTestBase):
             self.test22.name = "model2 instance2 version2"
             self.test22.save()
     
+    def testRevisionSignals(self):
+        pre_revision_receiver_called = []
+        def pre_revision_receiver(**kwargs):
+            self.assertEqual(kwargs["instances"], [self.test11])
+            self.assertTrue(isinstance(kwargs["revision"], Revision))
+            self.assertEqual(len(kwargs["versions"]), 1)
+            pre_revision_receiver_called.append(True)
+        post_revision_receiver_called = []
+        def post_revision_receiver(**kwargs):
+            self.assertEqual(kwargs["instances"], [self.test11])
+            self.assertTrue(isinstance(kwargs["revision"], Revision))
+            self.assertEqual(len(kwargs["versions"]), 1)
+            post_revision_receiver_called.append(True)
+        reversion.pre_revision_commit.connect(pre_revision_receiver)
+        reversion.post_revision_commit.connect(post_revision_receiver)
+        # Create a revision.
+        with reversion.create_revision():
+            self.test11.save()
+        # Check the signals were called.
+        self.assertTrue(pre_revision_receiver_called)
+        self.assertTrue(post_revision_receiver_called)
+    
     def testCanGetForObjectReference(self):
         # Test a model with an int pk.
         versions = reversion.get_for_object_reference(ReversionTestModel1, self.test11.pk)
@@ -276,15 +313,16 @@ class ApiTest(RevisionTestBase):
         self.assertEqual(len(reversion.get_unique_for_object(self.test21)), 2)
         
     def testCanGetForDate(self):
-        now = datetime.datetime.now()
-        # Test a model with an int pk.
-        version = reversion.get_for_date(self.test11, now)
-        self.assertEqual(version.field_dict["name"], "model1 instance1 version2")
-        self.assertRaises(Version.DoesNotExist, lambda: reversion.get_for_date(self.test11, datetime.datetime(1970, 1, 1)))
-        # Test a model with a str pk.
-        version = reversion.get_for_date(self.test21, now)
-        self.assertEqual(version.field_dict["name"], "model2 instance1 version2")
-        self.assertRaises(Version.DoesNotExist, lambda: reversion.get_for_date(self.test21, datetime.datetime(1970, 1, 1)))
+        with self.settings(USE_TZ=True):
+            now = datetime.datetime.now(UTC())
+            # Test a model with an int pk.
+            version = reversion.get_for_date(self.test11, now)
+            self.assertEqual(version.field_dict["name"], "model1 instance1 version2")
+            self.assertRaises(Version.DoesNotExist, lambda: reversion.get_for_date(self.test11, datetime.datetime(1970, 1, 1, tzinfo=UTC())))
+            # Test a model with a str pk.
+            version = reversion.get_for_date(self.test21, now)
+            self.assertEqual(version.field_dict["name"], "model2 instance1 version2")
+            self.assertRaises(Version.DoesNotExist, lambda: reversion.get_for_date(self.test21, datetime.datetime(1970, 1, 1, tzinfo=UTC())))
         
     def testCanGetDeleted(self):
         with reversion.create_revision():
@@ -674,7 +712,18 @@ class VersionAdminTest(TestCase):
         )
         self.user.set_password("bar")
         self.user.save()
-        self.client.login(username="foo", password="bar")
+        # Log the user in.
+        if hasattr(self, "settings"):
+            with self.settings(INSTALLED_APPS=tuple(set(tuple(settings.INSTALLED_APPS) + ("django.contrib.sessions",)))):  # HACK: Without this the client won't log in, for some reason.
+                self.client.login(
+                    username = "foo",
+                    password = "bar",
+                )
+        else:
+            self.client.login(
+                username = "foo",
+                password = "bar",
+            )
 
     @skipUnless('django.contrib.admin' in settings.INSTALLED_APPS,
                 "django.contrib.admin not activated")
